from lyzr.base.file_utils import read_file, describe_dataset
from lyzr.base.llm import LyzrLLMFactory
from lyzr.base.llms import LLM, Prompt, get_model
from lyzr.base.service import LyzrService
from lyzr.base.vector_store import LyzrVectorStoreIndex

__all__ = [
    "LyzrLLMFactory",
    "LyzrService",
    "LyzrVectorStoreIndex",
    "LLM",
    "Prompt",
    "get_model",
    "read_file",
    "describe_dataset",
]



# SPDX-FileCopyrightText: 2023-present patel <khush@base.ai>
#
# SPDX-License-Identifier: MIT

from lyzr.generator import Generator
from lyzr.chatqa.chatbot import ChatBot
from lyzr.base.llm import LyzrLLMFactory
from lyzr.chatqa.qa_bot import QABot
from lyzr.base.service import LyzrService
from lyzr.base.vector_store import LyzrVectorStoreIndex
from lyzr.formula_generator import FormulaGen

__all__ = [
    "LyzrLLMFactory",
    "LyzrService",
    "LyzrVectorStoreIndex",
    "QABot",
    "ChatBot",
    "FormulaGen",
    "Generator",
]



from lyzr.utils.document_reading import (
    read_pdf_as_documents,
    read_txt_as_documents,
    read_docx_as_documents,
    read_webpage_as_documents,
    read_website_as_documents,
    read_youtube_as_documents,
)

__all__ = [
    "read_pdf_as_documents",
    "read_txt_as_documents",
    "read_docx_as_documents",
    "read_webpage_as_documents",
    "read_website_as_documents",
    "read_youtube_as_documents",
]



from typing import List

from langchain.document_loaders import Docx2txtLoader
from llama_index.readers.base import BaseReader
from llama_index.schema import Document


class LyzrDocxReader(BaseReader):
    def __init__(self) -> None:
        None

    def load_data(self, file_path: str, extra_info: dict = None) -> List[Document]:
        loader = Docx2txtLoader(str(file_path))
        langchain_documents = loader.load()

        documents = []
        for langchain_document in langchain_documents:
            doc = Document.from_langchain_format(langchain_document)

            if extra_info is not None:
                doc.metadata.update(extra_info)

            documents.append(doc)

        return documents



import os
from importlib import resources as impresources
from typing import Optional

import openai

from . import prompts


class Prompt:
    def __init__(self, prompt_name: str, prompt_text: Optional[str] = None):
        self.name = prompt_name
        self.text = prompt_text
        if self.text is None:
            # in-built prompt names end with _pt
            self.load_prompt()
            self.variables = self.get_variables()
            return None
        else:
            self.variables = self.get_variables()
            self.save_prompt()
            return None

    def get_variables(self):
        variables = []
        for word in self.text.split():
            if word.startswith("{") and word.endswith("}"):
                variables.append(word[1:-1])
        return variables

    def save_prompt(self):
        inp_file = impresources.files(prompts) / f"{self.name}.txt"
        with inp_file.open("w+") as f:
            f.write(self.text.encode("utf-8"))

    def load_prompt(self):
        try:
            inp_file = impresources.files(prompts) / f"{self.name}.txt"
            with inp_file.open("rb") as f:
                self.text = f.read().decode("utf-8")
        except FileNotFoundError:
            raise ValueError(
                f"No prompt with name '{self.name}' found. "
                f"To use an in-built prompt, use one of the following prompt names: {get_prompts_list()}\n"
                "Or create a new prompt by passing the prompt text.",
            )

    def edit_prompt(self, prompt_text: str):
        self.text = prompt_text
        self.variables = self.get_variables()
        self.save_prompt()

    def format(self, **kwargs):
        if self.text is None:
            raise ValueError(f"Please provide the text for the prompt '{self.name}'")
        prompt_text = self.text
        try:
            prompt_text = prompt_text.format(**kwargs)
        except KeyError:
            print(f"Please provide values for all variables: {self.variables}")
            raise
        return prompt_text


class LLM:
    def __init__(
        self,
        api_key: str,
        model_type: Optional[str] = "openai",
        model_name: Optional[str] = "gpt-3.5-turbo",
        prompt_name: Optional[str] = None,
        prompt: Optional[Prompt] = None,
        **kwargs,
    ):
        self.api_key = api_key
        self.model_type = model_type
        self.model_name = model_name
        if prompt_name is not None:
            self.prompt = Prompt(prompt_name)
        elif prompt is not None:
            self.prompt = prompt
        for param in kwargs:
            setattr(self, param, kwargs[param])
        # llm_params = {
        #     "temperature": 0,
        #     "top_p": 1,
        #     "presence_penalty": 0,
        #     "frequency_penalty": 0,
        #     "stop": None,
        #     "n": 1,
        #     "stream": False,
        #     "functions": None,
        #     "function_call": None,
        #     "logit_bias": None,
        #     "best_of": 1,
        #     "echo": False,
        #     "logprobs": None,
        #     "suffix": None,
        # }
        # for param in llm_params:
        #     setattr(self, param, kwargs.get(param, llm_params[param]))

    def set_prompt(
        self,
        prompt_name: Optional[str] = None,
        prompt_text: Optional[str] = None,
        **kwargs,
    ):
        if prompt_name is None and self.prompt is None:
            raise ValueError("Please set a value for the prompt")

        if self.prompt is None:
            self.prompt = Prompt(prompt_name, prompt_text)

        self.prompt.text = self.prompt.format(**kwargs)

    def run(self, **kwargs):
        if self.api_key is None:
            raise ValueError(
                "Please provide an API key or set the API_KEY environment variable."
            )

        if "prompt" in kwargs:
            self.set_prompt(kwargs.pop("prompt"), **kwargs)

        empty_variables = self.prompt.get_variables()
        if empty_variables != []:
            raise ValueError(
                f"Please provide values for the following variables: {empty_variables}"
            )
        messages = [{"role": "system", "content": self.prompt.text}]

        params = self.__dict__.copy()
        for param in ["api_key", "model_type", "model_name", "prompt"]:
            del params[param]
        params.update(kwargs)

        if self.model_type == "openai":
            openai.api_key = self.api_key
            completion = openai.Completion.create(
                model=self.model_name,
                messages=messages,
                **params,
            )
            return completion


def get_model(
    api_key: Optional[str] = None,
    model_type: Optional[str] = None,
    model_name: Optional[str] = None,
) -> LLM:
    return LLM(
        api_key=api_key or os.getenv("API_KEY"),
        model_type=model_type or os.getenv("MODEL_TYPE") or "openai",
        model_name=model_name or os.getenv("MODEL_NAME") or "gpt-3.5-turbo",
    )


def get_prompts_list() -> list:
    # fix this path issue
    all_prompts = [
        pfile.stem
        for pfile in impresources.files(prompts).iterdir()
        if (pfile.suffix == ".txt") and (pfile.stem.endswith("_pt"))
    ]
    return all_prompts



from typing import Union, Optional, List

from llama_index import ServiceContext, VectorStoreIndex
from llama_index.embeddings.utils import EmbedType
from llama_index.indices.query.base import BaseQueryEngine

from lyzr.utils.rag_utils import (
    pdf_rag,
    txt_rag,
    docx_rag,
    webpage_rag,
    website_rag,
    youtube_rag,
)


class QABot:
    def __init__(self) -> None:
        return None

    @staticmethod
    def from_instances(
        vector_store_index: VectorStoreIndex, service_context: ServiceContext, **kwargs
    ) -> BaseQueryEngine:
        return vector_store_index.as_query_engine(
            service_context=service_context, **kwargs
        )

    @staticmethod
    def pdf_qa(
        input_dir: Optional[str] = None,
        input_files: Optional[List] = None,
        exclude_hidden: bool = True,
        filename_as_id: bool = True,
        recursive: bool = True,
        required_exts: Optional[List[str]] = None,
        system_prompt: str = None,
        query_wrapper_prompt: str = None,
        embed_model: Union[str, EmbedType] = "default",
        llm_params: dict = None,
        vector_store_params: dict = None,
        service_context_params: dict = None,
        query_engine_params: dict = None,
    ) -> BaseQueryEngine:
        return pdf_rag(
            input_dir=input_dir,
            input_files=input_files,
            exclude_hidden=exclude_hidden,
            filename_as_id=filename_as_id,
            recursive=recursive,
            required_exts=required_exts,
            system_prompt=system_prompt,
            query_wrapper_prompt=query_wrapper_prompt,
            embed_model=embed_model,
            llm_params=llm_params,
            vector_store_params=vector_store_params,
            service_context_params=service_context_params,
            query_engine_params=query_engine_params,
        )

    @staticmethod
    def docx_qa(
        input_dir: Optional[str] = None,
        input_files: Optional[List] = None,
        exclude_hidden: bool = True,
        filename_as_id: bool = True,
        recursive: bool = True,
        required_exts: Optional[List[str]] = None,
        system_prompt: str = None,
        query_wrapper_prompt: str = None,
        embed_model: Union[str, EmbedType] = "default",
        llm_params: dict = None,
        vector_store_params: dict = None,
        service_context_params: dict = None,
        query_engine_params: dict = None,
    ) -> BaseQueryEngine:
        return docx_rag(
            input_dir=input_dir,
            input_files=input_files,
            exclude_hidden=exclude_hidden,
            filename_as_id=filename_as_id,
            recursive=recursive,
            required_exts=required_exts,
            system_prompt=system_prompt,
            query_wrapper_prompt=query_wrapper_prompt,
            embed_model=embed_model,
            llm_params=llm_params,
            vector_store_params=vector_store_params,
            service_context_params=service_context_params,
            query_engine_params=query_engine_params,
        )

    @staticmethod
    def txt_qa(
        input_dir: Optional[str] = None,
        input_files: Optional[List] = None,
        exclude_hidden: bool = True,
        filename_as_id: bool = True,
        recursive: bool = True,
        required_exts: Optional[List[str]] = None,
        system_prompt: str = None,
        query_wrapper_prompt: str = None,
        embed_model: Union[str, EmbedType] = "default",
        llm_params: dict = None,
        vector_store_params: dict = None,
        service_context_params: dict = None,
        query_engine_params: dict = None,
    ) -> BaseQueryEngine:
        return txt_rag(
            input_dir=input_dir,
            input_files=input_files,
            exclude_hidden=exclude_hidden,
            filename_as_id=filename_as_id,
            recursive=recursive,
            required_exts=required_exts,
            system_prompt=system_prompt,
            query_wrapper_prompt=query_wrapper_prompt,
            embed_model=embed_model,
            llm_params=llm_params,
            vector_store_params=vector_store_params,
            service_context_params=service_context_params,
            query_engine_params=query_engine_params,
        )

    @staticmethod
    def webpage_qa(
        url: Optional[str] = None,
        system_prompt: str = None,
        query_wrapper_prompt: str = None,
        embed_model: Union[str, EmbedType] = "default",
        llm_params: dict = None,
        vector_store_params: dict = None,
        service_context_params: dict = None,
        query_engine_params: dict = None,
    ) -> BaseQueryEngine:
        return webpage_rag(
            url=url,
            system_prompt=system_prompt,
            query_wrapper_prompt=query_wrapper_prompt,
            embed_model=embed_model,
            llm_params=llm_params,
            vector_store_params=vector_store_params,
            service_context_params=service_context_params,
            query_engine_params=query_engine_params,
        )

    @staticmethod
    def website_qa(
        url: Optional[str] = None,
        system_prompt: str = None,
        query_wrapper_prompt: str = None,
        embed_model: Union[str, EmbedType] = "default",
        llm_params: dict = None,
        vector_store_params: dict = None,
        service_context_params: dict = None,
        query_engine_params: dict = None,
    ) -> BaseQueryEngine:
        return website_rag(
            url=url,
            system_prompt=system_prompt,
            query_wrapper_prompt=query_wrapper_prompt,
            embed_model=embed_model,
            llm_params=llm_params,
            vector_store_params=vector_store_params,
            service_context_params=service_context_params,
            query_engine_params=query_engine_params,
        )

    @staticmethod
    def youtube_qa(
        urls: List[str] = None,
        system_prompt: str = None,
        query_wrapper_prompt: str = None,
        embed_model: Union[str, EmbedType] = "default",
        llm_params: dict = None,
        vector_store_params: dict = None,
        service_context_params: dict = None,
        query_engine_params: dict = None,
    ) -> BaseQueryEngine:
        return youtube_rag(
            urls=urls,
            system_prompt=system_prompt,
            query_wrapper_prompt=query_wrapper_prompt,
            embed_model=embed_model,
            llm_params=llm_params,
            vector_store_params=vector_store_params,
            service_context_params=service_context_params,
            query_engine_params=query_engine_params,
        )



from setuptools import setup, find_packages

setup(
    name="lyzr",
    version="0.1.6",
    author="lyzr",
    description="",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="",
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8.1, <3.12",
    install_requires=[
        "asyncio",
        "nest_asyncio",
        "openai==0.28.1",
        "litellm==0.12.8",
        "llama-index==0.8.57",
        "langchain==0.0.329",
        "python-dotenv>=1.0.0",
        "tiktoken>=0.5.1",
        "pdfminer-six==20221105",
        "lancedb==0.3.3",
        "beautifulsoup4==4.12.2",
        "docx2txt==0.8",
        "playwright==1.39.0",
        "pytest-playwright==0.4.3",
        "pandas==2.1.2",
    ],
)



import logging

from lyzr.utils.templates import QUERY_PROMPT_TEMPLATE, SYSTEM_PROMPT

logger = logging.getLogger(__name__)


def set_default_prompt_template():
    return SYSTEM_PROMPT, QUERY_PROMPT_TEMPLATE



from typing import Optional, Sequence

from llama_index import Document, ServiceContext, VectorStoreIndex, StorageContext
from llama_index.node_parser import SimpleNodeParser
from llama_index.node_parser.extractors import (
    EntityExtractor,
    KeywordExtractor,
    MetadataExtractor,
    QuestionsAnsweredExtractor,
    SummaryExtractor,
    TitleExtractor,
)


def import_vector_store_class(vector_store_class_name: str):
    module = __import__("llama_index.vector_stores", fromlist=[vector_store_class_name])
    class_ = getattr(module, vector_store_class_name)
    return class_


class LyzrVectorStoreIndex:
    @staticmethod
    def from_defaults(
            vector_store_type: str = "LanceDBVectorStore",
            enable_metadata_extraction: bool = False,
            documents: Optional[Sequence[Document]] = None,
            service_context: Optional[ServiceContext] = None,
            **kwargs
    ) -> VectorStoreIndex:

        if documents is None and vector_store_type == "SimpleVectorStore":
            raise ValueError("documents must be provided for SimpleVectorStore")

        vector_store_class = import_vector_store_class(vector_store_type)

        if documents is None:
            vector_store = vector_store_class(**kwargs)
            index = VectorStoreIndex.from_vector_store(
                vector_store=vector_store, service_context=service_context
            )
        else:
            if vector_store_type == "LanceDBVectorStore":
                kwargs["uri"] = "./.lancedb" if "uri" not in kwargs else kwargs["uri"]
                kwargs["table_name"] = (
                    "vectors" if "table_name" not in kwargs else kwargs["table_name"]
                )
            vector_store = vector_store_class(**kwargs)
            storage_context = StorageContext.from_defaults(vector_store=vector_store)

            llm = service_context.llm if service_context is not None else None

            if enable_metadata_extraction:
                metadata_extractor = MetadataExtractor(
                    extractors=[
                        TitleExtractor(llm=llm, nodes=5),
                        QuestionsAnsweredExtractor(llm=llm, questions=3),
                        SummaryExtractor(llm=llm, summaries=["prev", "self"]),
                        KeywordExtractor(llm=llm, keywords=10),
                        EntityExtractor(prediction_threshold=0.5),
                    ],
                )
                node_parser = SimpleNodeParser.from_defaults(
                    metadata_extractor=metadata_extractor
                )
                nodes = node_parser.get_nodes_from_documents(documents)
                index = VectorStoreIndex(
                    nodes=nodes,
                    storage_context=storage_context,
                    service_context=service_context,
                    show_progress=True,
                )
            else:
                index = VectorStoreIndex.from_documents(
                    documents=documents,
                    storage_context=storage_context,
                    service_context=service_context,
                    show_progress=True,
                )

        return index
